#!/bin/sh

# execute the sql in the schema files to create the database objects
#
# note that this script only handles loading the objects into an
# existing database.  See 'man createdb' and 'man dropdb'.
#
# you'll need to set the PGHOST, PGUSER, and PGPASSWORD environmental
# variables to make this script work correctl

if [ -z $2 ]; then
    echo "usage: run-sql-scripts.sh <schema directory> <db> ";
    exit;
fi

if [ "$PGUSER" == "" ]; then
    echo "You must set the PGUSER environment variable";
    exit;
fi
if [ "$PGPASSWORD" == "" ]; then
    echo "You can set the PGPASSWORD environment variable to avoid multiple password prompts";
fi

DIRECTORY=$1
DB=$2

PRESQL=`find $1 -name '*.pre.sql' -print`
SQL=`find $1 -name '*.sql' ! -name '*.post.sql' ! -name '*.pre.sql' -print`
POSTSQL=`find $1 -name '*.post.sql' -print`

TMP=`mktemp /tmp/run-sql-scripts.XXXXXX`;

for f in $PRESQL $SQL $POSTSQL; do
    echo `basename $f`
    psql -q -d "$DB" -f "$f" &> $TMP
    egrep -v '(NOTICE|function plpgsql_call_handler already exists|Language plpgsql already exists)' $TMP
done

rm $TMP
