/* $Id: functions.pre.sql,v 1.12 2004/01/28 01:37:46 hroberts Exp $
   Copyright 2002 Berkman Center for Internet & Society. All Rights Reserved.
 */

/* postgresql magic to setup the plpgsql stuff */
create or replace function plpgsql_call_handler () 
       returns opaque as '/usr/lib/pgsql/plpgsql.so' language 'C';
create TRUSTED procedural language 'plpgsql' 
       handler plpgsql_call_handler lancompiler 'PL/pgSQL';

create or replace function requests_unique()
       returns opaque
       as '
       DECLARE 
           duplicate_requests int;
       BEGIN 
           if (new.status = 1) then
               select into duplicate_requests count(*) from requests
                      where requests_id <> new.requests_id and 
                            status = 1 and
                            position = new.position and
                            ((joiner_projects_id is null and 
                              new.joiner_projects_id is null) or
                             (joiner_projects_id = new.joiner_projects_id)) and
                            ((joiner_users_id is null and 
                              new.joiner_users_id is null) or
                             (joiner_users_id = new.joiner_users_id)) and
                            ((activity_projects_id is null and 
                              new.activity_projects_id is null) or
                             (activity_projects_id = 
                              new.activity_projects_id)) and
                            ((activity_rotisseries_id is null and 
                              new.activity_rotisseries_id is null) or
                             (activity_rotisseries_id = 
                              new.activity_rotisseries_id)) and
                            ((activity_board_thread_id is null and
                              new.activity_board_thread_id is null) or
                             (activity_board_thread_id = 
                              new.activity_board_thread_id));
               if (duplicate_requests > 0) then
                   raise exception ''Cannot insert a duplicate key into requests: requests_unique trigger check failed'';
               end if;
           end if;
           return new;
       END;'
       language 'plpgsql';

create or replace function insert_comment()
       returns opaque
       as '
       DECLARE
           parent_comment record;
       BEGIN 
           if (new.root_comments_id is not null) then
               raise exception ''root_comments_id must be null'';
               return null;
           end if;
           if (new.parent_comments_id is null) then
               new.root_comments_id = new.comments_id;
           else
               for parent_comment in select * from comments
                   where comments_id = new.parent_comments_id limit 1 loop
                   
                   new.root_comments_id = parent_comment.root_comments_id;
               end loop;
           end if;
           return new;
       END;'
       language 'plpgsql';

create or replace function delete_comment()
       returns opaque
       as '
       DECLARE
           new_root_comments_id int;
       BEGIN 
           if (old.parent_comments_id is null) then
               select into new_root_comments_id comments_id from comments
                      where parent_comments_id = old.comments_id;
               if (new_root_comments_id is not null) then
                   update comments set parent_comments_id = null 
                          where comments_id = new_root_comments_id;
                   update comments set root_comments_id = new_root_comments_id
                          where root_comments_id = old.root_comments_id;
               end if;
           else
               update comments set parent_comments_id = old.parent_comments_id 
                      where parent_comments_id = old.comments_id;
           end if;
           return old;
       END;'
       language 'plpgsql';

create or replace function insert_user_project_name()
       returns opaque
       as '
       DECLARE
           project_name varchar(128);
       BEGIN
           if (new.namespace_projects_id is not null) then
              select into project_name name from projects
                     where projects_id = new.namespace_projects_id;
              new.namespace_projects_name = project_name;
           end if;
           return new;
       END;'
       language 'plpgsql';

create or replace function update_user_project_name()
       returns opaque
       as '
       DECLARE
           project_name varchar(128);
       BEGIN
           if (new.namespace_projects_id is null) then
               new.namespace_projects_name = null;
           elsif ((old.namespace_projects_id is null) or (old.namespace_projects_id <> new.namespace_projects_id)) then
               select into project_name name from projects
                      where projects_id = new.namespace_projects_id;
               new.namespace_projects_name = project_name;
           end if;
           return new;
       END;'
       language 'plpgsql';

create or replace function update_project_name()
       returns opaque
       as '
       BEGIN
           if (new.name <> old.name) then
               update users set namespace_projects_name = new.name
                      where namespace_projects_id = new.projects_id;
           end if;
           return new;
       END;'
       language 'plpgsql';

create or replace function check_user_name_unique()
       returns opaque
       as '
       DECLARE
           duplicate_users int;
       BEGIN
           if (new.namespace_projects_id is null) then
               select into duplicate_users users_id from users
                      where lower_name = lower(new.name) and
                      users_id <> new.users_id and
                      namespace_projects_id is null;
           else 
               select into duplicate_users users_id from users
                      where lower_name = lower(new.name) and
                      users_id <> new.users_id and
                      namespace_projects_id = new.namespace_projects_id;
           end if;
           if (duplicate_users) then
               raise exception ''Cannot insert a duplicate key into requests: check_user_name_unique() trigger check failed'';
           end if;
           return new;
       END;'
       language 'plpgsql';


               