/* $Id: functions.pre.sql,v 1.1 2003/03/04 19:40:48 jmarinus Exp $
   Copyright 2002 Berkman Center for Internet & Society. All Rights Reserved.
 */

/* postgresql magic to setup the plpgsql stuff */
create or replace function plpgsql_call_handler () 
       returns opaque as '/usr/lib/pgsql/plpgsql.so' language 'C';
create TRUSTED procedural language 'plpgsql' 
       handler plpgsql_call_handler lancompiler 'PL/pgSQL';

create or replace function insert_post()
       returns opaque
       as '
       DECLARE
           parent_post record;
       BEGIN 
           if (new.root_posts_id is not null) then
               raise exception ''root_posts_id must be null'';
               return null;
           end if;
           if (new.parent_posts_id is null) then
               new.root_posts_id = new.posts_id;
               new.thread_rating_total = new.rating;
               new.thread_count = 1;
           else
               for parent_post in select * from posts
                   where posts_id = new.parent_posts_id limit 1 loop
                   
                   new.root_posts_id = parent_post.root_posts_id;
                   new.thread_rating_total = parent_post.thread_rating_total;
                   new.thread_count = parent_post.thread_count;
               end loop;
               new.thread_rating_total = new.thread_rating_total + new.rating;
               new.thread_count = new.thread_count + 1;
               update posts set 
                            thread_rating_total = new.thread_rating_total,
                            thread_count = new.thread_count
                      where root_posts_id = new.root_posts_id;
           end if;
           return new;
       END;'
       language 'plpgsql';

create or replace function update_post()
       returns opaque
       as '    
       BEGIN
           if (new.rating <> old.rating) then
               update posts set thread_rating_total = 
                      thread_rating_total + (new.rating - old.rating)
                   where root_posts_id = old.root_posts_id;
           end if;
           return new;
       END;'
       language 'plpgsql';    

create or replace function delete_post()
       returns opaque
       as '
       DECLARE
           new_root_posts_id int;
       BEGIN 
           update posts 
                  set thread_rating_total = 
                      thread_rating_total - old.rating,
                      thread_count = thread_count - 1
                  where root_posts_id = old.root_posts_id;
           if (old.parent_posts_id is null) then
               select into new_root_posts_id posts_id from posts
                      where parent_posts_id = old.posts_id;
               if (new_root_posts_id is not null) then
                   update posts set parent_posts_id = null 
                          where posts_id = new_root_posts_id;
                   update posts set root_posts_id = new_root_posts_id
                          where root_posts_id = old.root_posts_id;
               end if;
           else
               update posts set parent_posts_id = old.parent_posts_id 
                      where parent_posts_id = old.posts_id;
           end if;
           return old;
       END;'
       language 'plpgsql';

create or replace function update_rotisserie_start_date()
       returns opaque
       as '
       DECLARE
           rnd record;
           last_collect bigint;
           new_collect bigint;
       BEGIN   
           last_collect = new.start_date;

           if (old.start_date <> new.start_date) then
           for rnd in select * from rounds 
                               where rotisseries_id = old.rotisseries_id 
                               order by round_order asc loop

               new_collect := rnd.due_date + 
                              get_grace_period(rnd.use_grace_period, last_collect, rnd.due_date);

               update rounds 
                      set route_date = last_collect,
                          send_date = last_collect,
                          collect_date = new_collect
                      where rounds_id = rnd.rounds_id;

               last_collect := new_collect;
           end loop;
           end if;

           return new;
        END;'
        language 'plpgsql';

/* loops through all the rounds, setting the route/send/collect dates.
   this trigger should be "after insert", since we want to include
   the new round in our query loop. */
create or replace function insert_round_due_date()
       returns opaque
       as '
       DECLARE
           rnd record;
           last_collect bigint;
           new_collect bigint;
       BEGIN   
           select into last_collect start_date
                  from rotisseries where rotisseries_id = new.rotisseries_id;

           for rnd in select * from rounds 
                               where rotisseries_id = new.rotisseries_id 
                               order by round_order asc loop

               new_collect := rnd.due_date + 
                              get_grace_period(rnd.use_grace_period, last_collect, rnd.due_date);
               update rounds 
                      set route_date = last_collect,
                          send_date = last_collect,
                          collect_date = new_collect
                      where rounds_id = rnd.rounds_id;

               last_collect := new_collect;
           end loop;

           return new;
        END;'
        language 'plpgsql';

/* loops through rounds after this round, setting the route/send/collect dates. */
create or replace function update_round_due_date()
       returns opaque
       as '
       DECLARE
           rnd record;
           last_collect bigint;
           new_collect bigint;
       BEGIN   
           if ((new.due_date <> old.due_date) or
               (new.use_grace_period <> old.use_grace_period) or
               (new.round_order <> old.round_order)) then
               select into last_collect start_date
                      from rotisseries where rotisseries_id = new.rotisseries_id;
               for rnd in select * from rounds 
                                   where rotisseries_id = new.rotisseries_id 
                                   order by round_order asc loop
                   new_collect := rnd.due_date + 
                                  get_grace_period(rnd.use_grace_period, last_collect, rnd.due_date);

                   update rounds 
                      set route_date = last_collect,
                          send_date = last_collect,
                          collect_date = new_collect
                      where rounds_id = rnd.rounds_id;

                   last_collect := new_collect;
               end loop;

           end if;
           return new;
        END;'
        language 'plpgsql';


/* loops through all the rounds, setting the route/send/collect dates. */
create or replace function delete_round_due_date()
       returns opaque
       as '
       DECLARE
           rnd record;
           last_collect bigint;
           new_collect bigint;
       BEGIN   
           select into last_collect start_date
                  from rotisseries where rotisseries_id = old.rotisseries_id;

           for rnd in select * from rounds 
                               where rotisseries_id = old.rotisseries_id 
                               order by round_order asc loop
               new_collect := rnd.due_date + 
                              get_grace_period(rnd.use_grace_period, last_collect, rnd.due_date);

               update rounds 
                   set route_date = last_collect,
                       send_date = last_collect,
                       collect_date = new_collect
                   where rounds_id = rnd.rounds_id;

               last_collect := new_collect;
           end loop;

           return new;
        END;'
        language 'plpgsql';

create or replace function insert_poll_choice_user()
       returns opaque
       as '
       DECLARE
           vote_count int;
       BEGIN 
           select into vote_count votes from polls_choices
                                  where polls_choices_id = new.polls_choices_id;
           vote_count := vote_count + 1;
           update polls_choices 
                  set votes = vote_count
                  where polls_choices_id = new.polls_choices_id; 
           return new;
       END;'
       language 'plpgsql';

create or replace function update_poll_choice_user()
       returns opaque
       as '
       DECLARE
           vote_count int;
       BEGIN 
           if (new.polls_choices_id <> old.polls_choices_id) then
               select into vote_count votes from polls_choices
                                      where polls_choices_id = old.polls_choices_id;
               vote_count := vote_count - 1;
               update polls_choices 
                      set votes = vote_count
                      where polls_choices_id = old.polls_choices_id; 
               select into vote_count votes from polls_choices
                                      where polls_choices_id = new.polls_choices_id;
               vote_count := vote_count + 1;
               update polls_choices 
                      set votes = vote_count
                      where polls_choices_id = new.polls_choices_id; 
           end if;
           return new;
       END;'
       language 'plpgsql';

create or replace function delete_poll_choice_user()
       returns opaque
       as '
       DECLARE
           vote_count int;
       BEGIN 
           select into vote_count votes from polls_choices
                                  where polls_choices_id = old.polls_choices_id;
           vote_count := vote_count - 1;
           update polls_choices 
                  set votes = vote_count
                  where polls_choices_id = old.polls_choices_id; 
           return old;
       END;'
       language 'plpgsql';

/* computes the difference between the due date and the collect date,
   which is the grace period.  the formula calculates a grace period
   of 10% of the round duration, with a 1 minute minimum and 10 minute
   maximum if use_grace_period is true.  if use_grace_period is false,
   the collect date is the same as the due date.
   arg1: use grace period, false always returns 0
   arg2: round route date
   arg3: round due date
   returns: grace period */
create or replace function get_grace_period(boolean, bigint, bigint) 
       returns bigint 
       as '
       DECLARE
           gp bigint := 0;
       BEGIN 
           gp := trunc(($3 - $2) * 0.10);
           if $1 = FALSE then
               return 0;
           elsif (gp < 60000) then
               return 60000;           
	   elsif (gp > 600000) then
               return 600000;
           else 
               return gp;
           end if;
       END;' 
       language plpgsql
       with (iscachable);
